@@ -12,10 +12,6 @@ Sometime when you want to custome x509, you maybe need to use this.
 #include "private.h"
 #include <openssl/asn1.h>
 
-#ifdef WIN32
-#define timezone _timezone
-#endif
-
 static LuaL_Enumeration asn1_const[] =
 {
   /* 0 */
@@ -307,7 +303,7 @@ static int openssl_asn1string_new(lua_State* L)
 {
   size_t size = 0;
   const char* data = luaL_checklstring(L, 1, &size);
-  int type = luaL_checkint(L, 2);
+  int type = luaL_optint(L, 2, V_ASN1_UTF8STRING);
   ASN1_STRING *s = ASN1_STRING_type_new(type);
   ASN1_STRING_set(s, data, size);
   PUSH_OBJECT(s, "openssl.asn1_string");
@@ -1008,17 +1004,32 @@ static int openssl_asn1group_set(lua_State *L)
     return openssl_pushresult(L, ret);
   }
   case V_ASN1_UTCTIME:
+  {
+    ASN1_TIME *a = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
+    if (lua_type(L, 2) == LUA_TNUMBER)
+    {
+      time_t t = luaL_checkinteger(L, 2);
+      ASN1_UTCTIME_set(a, t);
+    }
+    else if (lua_isstring(L, 2))
+    {
+      ret = ASN1_UTCTIME_set_string(a, lua_tostring(L, 2));
+    }
+    else
+      luaL_error(L, "only accpet number or string");
+    return openssl_pushresult(L, ret);
+  }
   case V_ASN1_GENERALIZEDTIME:
   {
     ASN1_TIME *a = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
     if (lua_type(L, 2) == LUA_TNUMBER)
     {
       time_t t = luaL_checkinteger(L, 2);
-      ASN1_TIME_set(a, t);
+      ASN1_GENERALIZEDTIME_set(a, t);
     }
     else if (lua_isstring(L, 2))
     {
-      ret = ASN1_TIME_set_string(a, lua_tostring(L, 2));
+      ret = ASN1_GENERALIZEDTIME_set_string(a, lua_tostring(L, 2));
     }
     else
       luaL_error(L, "only accpet number or string");
@@ -1087,8 +1098,7 @@ static int openssl_asn1group_get(lua_State *L)
   case V_ASN1_GENERALIZEDTIME:
   {
     ASN1_TIME *at = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
-    time_t offset = timezone;
-    time_t get = ASN1_TIME_get(at, -offset);
+    time_t get = ASN1_TIME_get(at, 0);
     lua_pushnumber(L, (lua_Number) get);
     return 1;
   }
@@ -1428,15 +1438,47 @@ static int openssl_asn1time_check(lua_State* L)
 
 static int openssl_asn1time_adj(lua_State* L)
 {
-  ASN1_TIME *a = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
+  ASN1_TIME *at = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
   time_t t = luaL_checkinteger(L, 2);
   int offset_day = luaL_optint(L, 3, 0);
   long offset_sec = luaL_optlong(L, 4, 0);
 
-  ASN1_TIME_adj(a, t, offset_day, offset_sec);
-  return 0;
+  switch(at->type)
+  {
+  case V_ASN1_UTCTIME:
+  {
+    ASN1_UTCTIME *a = (ASN1_UTCTIME*)at;
+    ASN1_UTCTIME_adj(a, t, offset_day, offset_sec);
+    break;
+  }
+  case V_ASN1_GENERALIZEDTIME:
+  {
+    ASN1_GENERALIZEDTIME *a = (ASN1_UTCTIME*)at;
+    ASN1_GENERALIZEDTIME_adj(a, t, offset_day, offset_sec);
+    break;
+  }
+  }
+  lua_pushvalue(L, 1);
+  return 1;
 }
 
+#if !defined(LIBRESSL_VERSION_NUMBER)
+static int openssl_asn1time_diff(lua_State* L)
+{
+  int day, sec, ret;
+  ASN1_TIME *from = CHECK_OBJECT(1, ASN1_TIME, "openssl.asn1_time");
+  ASN1_TIME *to = lua_isnoneornil(L, 2) ? NULL
+                  : CHECK_OBJECT(2, ASN1_TIME, "openssl.asn1_time");
+
+  ret = ASN1_TIME_diff(&day, &sec, from, to);
+  if (ret==0)
+    return openssl_pushresult(L, ret);
+  lua_pushinteger(L, day);
+  lua_pushinteger(L, sec);
+  return 2;
+}
+#endif
+
 static luaL_Reg asn1str_funcs[] =
 {
   /* asn1string */
@@ -1466,6 +1508,9 @@ static luaL_Reg asn1str_funcs[] =
 
   /* asn1time,asn1generalizedtime */
   {"adj", openssl_asn1time_adj},
+#if !defined(LIBRESSL_VERSION_NUMBER)
+  {"diff", openssl_asn1time_diff},
+#endif
   {"check", openssl_asn1time_check},
 
   {NULL, NULL}
@@ -1473,7 +1518,6 @@ static luaL_Reg asn1str_funcs[] =
 
 int luaopen_asn1(lua_State *L)
 {
-  tzset();
   auxiliar_newclass(L, "openssl.asn1_object", asn1obj_funcs);
   auxiliar_newclass(L, "openssl.asn1_type", asn1type_funcs);
 
@@ -1573,3 +1617,15 @@ int openssl_push_asn1(lua_State* L, const ASN1_STRING* string, int type)
   }
 
 }
+
+int openssl_push_asn1integer_as_bn(lua_State *L, const ASN1_INTEGER* ai)
+{
+  if(ai!=NULL)
+  {
+    BIGNUM *bn;
+    bn = ASN1_INTEGER_to_BN(ai, NULL);
+    PUSH_OBJECT(bn, "openssl.bn");
+  } else
+    lua_pushnil(L);
+  return 1;
+}
