@@ -686,33 +686,15 @@ static LUA_FUNCTION(openssl_evp_cipher_update)
   int outl;
   char *out;
   CIPHER_MODE mode;
-  int ret, type, nout;
+  int ret, type;
 
   EVP_CIPHER_CTX* c = CHECK_OBJECT(1, EVP_CIPHER_CTX, "openssl.evp_cipher_ctx");
   type = lua_type(L, 2);
   luaL_argcheck(L, type==LUA_TNUMBER || type==LUA_TSTRING, 2, "expect integer or string");
 
-  if (type==LUA_TNUMBER)
-  {
-    inl = lua_tointeger(L, 2);
-    in = NULL;
-    nout = 1;
-  }
-  else
-  {
-    in = luaL_checklstring(L, 2, &inl);
-    nout = lua_isnone(L, 3) ? 0 : lua_toboolean(L, 3);
-  }
-  if (nout==0)
-  {
-    outl = inl + EVP_MAX_BLOCK_LENGTH;
-    out = OPENSSL_malloc(outl);
-  }
-  else
-  {
-    outl = 0;
-    out = NULL;
-  }
+  in = luaL_checklstring(L, 2, &inl);
+  outl = inl + EVP_MAX_BLOCK_LENGTH;
+  out = OPENSSL_malloc(outl);
 
   lua_rawgetp(L, LUA_REGISTRYINDEX, c);
   mode = lua_tointeger(L, -1);
@@ -729,16 +711,11 @@ static LUA_FUNCTION(openssl_evp_cipher_update)
     luaL_error(L, "never go here");
 
   if (ret == 1)
-  {
-    if (out == NULL)
-      lua_pushinteger(L, outl);
-    else
-      lua_pushlstring(L, out, outl);
-  }
+    lua_pushlstring(L, out, outl);
   else
     ret = openssl_pushresult(L, ret);
-  if(out)
-    OPENSSL_free(out);
+
+  OPENSSL_free(out);
 
   return ret;
 }
@@ -813,7 +790,7 @@ static LUA_FUNCTION(openssl_cipher_ctx_padding)
 
 static LUA_FUNCTION(openssl_cipher_ctx_ctrl)
 {
-  int ret;
+  int ret = 0;
   EVP_CIPHER_CTX *ctx = CHECK_OBJECT(1, EVP_CIPHER_CTX, "openssl.evp_cipher_ctx");
   int type = luaL_checkint(L, 2);
   int arg = 0;
@@ -823,7 +800,7 @@ static LUA_FUNCTION(openssl_cipher_ctx_ctrl)
   {
   case EVP_CTRL_INIT:
     ret = EVP_CIPHER_CTX_ctrl(ctx, type, 0, NULL);
-    return openssl_pushresult(L, ret);
+    ret = openssl_pushresult(L, ret);
     break;
   case EVP_CTRL_SET_KEY_LENGTH:
   case EVP_CTRL_SET_RC2_KEY_BITS:
@@ -831,35 +808,28 @@ static LUA_FUNCTION(openssl_cipher_ctx_ctrl)
   case EVP_CTRL_GCM_SET_IVLEN:  //EVP_CTRL_CCM_SET_IVLEN
     arg = luaL_checkint(L, 3);
     ret = EVP_CIPHER_CTX_ctrl(ctx, type, arg, NULL);
-    return openssl_pushresult(L, ret);
+    ret = openssl_pushresult(L, ret);
     break;
   case EVP_CTRL_GCM_SET_TAG:    //EVP_CTRL_CCM_SET_TAG
   {
+    size_t sz = 0;
     luaL_argcheck(L, lua_isnumber(L, 3) || lua_isstring(L, 3), 3, "need integer or string");
-    if (lua_isnumber(L, 3))
-    {
-      arg = lua_tointeger(L, 3);
-      ret = EVP_CIPHER_CTX_ctrl(ctx, type, arg, ptr);
-    }
-    else
-    {
-      size_t sz = 0;
-      ptr = (void*)luaL_checklstring(L, 3, &sz);
-      arg = (int)sz;
-      ret = EVP_CIPHER_CTX_ctrl(ctx, type, arg, ptr);
-    }
-    return openssl_pushresult(L, ret);
+
+    ptr = (void*)luaL_checklstring(L, 3, &sz);
+    ret = EVP_CIPHER_CTX_ctrl(ctx, type, sz, ptr);
+
+    ret = openssl_pushresult(L, ret);
     break;
   }
   case EVP_CTRL_GET_RC2_KEY_BITS:
   case EVP_CTRL_GET_RC5_ROUNDS:
     ret = EVP_CIPHER_CTX_ctrl(ctx, type, 0, &arg);
-    if(ret==0)
+    if(ret==1)
     {
       lua_pushinteger(L, arg);
-      return 1;
-    }
-    return openssl_pushresult(L, ret);
+      ret = 1;
+    }else
+      ret = openssl_pushresult(L, ret);
   case EVP_CTRL_GCM_GET_TAG:    //EVP_CTRL_CCM_GET_TAG
   {
     char buf[16];
@@ -870,9 +840,10 @@ static LUA_FUNCTION(openssl_cipher_ctx_ctrl)
       if(ret==1)
       {
         lua_pushlstring(L, buf, arg);
-        return 1;
+        ret = 1;
       }
-      return openssl_pushresult(L, ret);
+      else 
+        ret = openssl_pushresult(L, ret);
     }
     else
       luaL_argerror(L, 3, "invalid integer, must be 4, 6, 10, 12, 14 or 16");
@@ -897,7 +868,7 @@ static LUA_FUNCTION(openssl_cipher_ctx_ctrl)
   default:
     luaL_error(L, "not support");
   }
-  return 0;
+  return ret;
 }
 
 static LUA_FUNCTION(openssl_cipher_ctx_free)
