@@ -22,10 +22,7 @@ static LUA_FUNCTION(openssl_dh_parse)
   DH* dh = CHECK_OBJECT(1, DH, "openssl.dh");
   lua_newtable(L);
 
-#if (OPENSSL_VERSION_NUMBER < 0x30000000L)
   lua_pushinteger(L, DH_size(dh));
-#endif
-
   lua_setfield(L, -2, "size");
 
   lua_pushinteger(L, DH_bits(dh));
@@ -43,19 +40,22 @@ static LUA_FUNCTION(openssl_dh_parse)
   return 1;
 }
 
-static int openssl_dh_set_engine(lua_State *L)
+static LUA_FUNCTION(openssl_dh_check)
 {
-#if !defined(OPENSSL_NO_ENGINE) && OPENSSL_VERSION_NUMBER < 0x30000000L
-  DH* dh = CHECK_OBJECT(1, DH, "openssl.dh");
-  ENGINE *e = CHECK_OBJECT(2, ENGINE, "openssl.engine");
-  const DH_METHOD *m = ENGINE_get_DH(e);
-  if (m)
+  const DH* dh = CHECK_OBJECT(1, DH, "openssl.dh");
+  int ret = 0;
+  int codes = 0;
+
+  if (lua_isuserdata(L, 2))
   {
-    int r = DH_set_method(dh, m);
-    return openssl_pushresult(L, r);
-  }
-#endif
-  return 0;
+    const BIGNUM* pub = CHECK_OBJECT(2, BIGNUM, "openssl.bn");
+    ret = DH_check_pub_key(dh, pub, &codes);
+  } else
+    ret = DH_check(dh, &codes);
+
+  lua_pushboolean(L, ret);
+  lua_pushinteger(L, codes);
+  return 2;
 }
 
 static int openssl_dh_generate_parameters(lua_State *L)
@@ -85,8 +85,6 @@ static int openssl_dh_generate_key(lua_State *L)
   int ret = DH_generate_key(dh);
   if (ret == 1)
   {
-    //TODO: remove up_ref
-    DH_up_ref(dh);
     PUSH_OBJECT(dh, "openssl.dh");
     return 1;
   }
@@ -98,9 +96,7 @@ static luaL_Reg dh_funs[] =
 {
   {"generate_key",  openssl_dh_generate_key},
   {"parse",         openssl_dh_parse},
-#if !defined(OPENSSL_NO_ENGINE) && OPENSSL_VERSION_NUMBER < 0x30000000L
-  {"set_engine",    openssl_dh_set_engine},
-#endif
+  {"check",         openssl_dh_check},
 
   {"__gc",          openssl_dh_free},
   {"__tostring",    auxiliar_tostring},
@@ -108,13 +104,81 @@ static luaL_Reg dh_funs[] =
   { NULL, NULL }
 };
 
+static LuaL_Enumeration dh_problems[] =
+{
+  {"DH_CHECK_P_NOT_PRIME",         DH_CHECK_P_NOT_PRIME},
+  {"DH_CHECK_P_NOT_SAFE_PRIME",    DH_CHECK_P_NOT_SAFE_PRIME},
+  {"DH_UNABLE_TO_CHECK_GENERATOR", DH_UNABLE_TO_CHECK_GENERATOR},
+  {"DH_NOT_SUITABLE_GENERATOR",    DH_NOT_SUITABLE_GENERATOR},
+#ifdef DH_CHECK_Q_NOT_PRIME
+  {"DH_CHECK_Q_NOT_PRIME",         DH_CHECK_Q_NOT_PRIME},
+#endif
+#ifdef DH_CHECK_INVALID_Q_VALUE
+  {"DH_CHECK_INVALID_Q_VALUE",     DH_CHECK_INVALID_Q_VALUE},
+#endif
+#ifdef DH_CHECK_INVALID_J_VALUE
+  {"DH_CHECK_INVALID_J_VALUE",     DH_CHECK_INVALID_J_VALUE},
+#endif
+
+  {"DH_CHECK_PUBKEY_TOO_SMALL",    DH_CHECK_PUBKEY_TOO_SMALL},
+  {"DH_CHECK_PUBKEY_TOO_LARGE",    DH_CHECK_PUBKEY_TOO_LARGE},
+#ifdef DH_CHECK_PUBKEY_INVALID
+  {"DH_CHECK_PUBKEY_INVALID",      DH_CHECK_PUBKEY_INVALID},
+#endif
+
+  {NULL,                           -1}
+};
+
+static int openssl_dh_problems(lua_State *L)
+{
+  int reason = luaL_checkint(L, 1);
+  int pub = lua_toboolean(L, 2);
+  int i = 1;
+
+#define VAL(r, v)  if(r & DH_##v) \
+  { lua_pushliteral(L, #v);  lua_rawseti(L, -2, i++); }
+
+  lua_newtable(L);
+  if (pub)
+  {
+    VAL(reason, CHECK_PUBKEY_TOO_SMALL);
+    VAL(reason, CHECK_PUBKEY_TOO_LARGE);
+#ifdef DH_CHECK_PUBKEY_INVALID
+    VAL(reason, CHECK_PUBKEY_INVALID);
+#endif
+  }
+  else
+  {
+    VAL(reason, CHECK_P_NOT_PRIME);
+    VAL(reason, CHECK_PUBKEY_TOO_SMALL);
+    VAL(reason, UNABLE_TO_CHECK_GENERATOR);
+    VAL(reason, NOT_SUITABLE_GENERATOR);
+
+#ifdef DH_CHECK_Q_NOT_PRIME
+    VAL(reason, CHECK_Q_NOT_PRIME);
+#endif
+#ifdef DH_CHECK_INVALID_Q_VALUE
+    VAL(reason, CHECK_INVALID_Q_VALUE);
+#endif
+#ifdef DH_CHECK_INVALID_J_VALUE
+    VAL(reason, CHECK_INVALID_J_VALUE);
+#endif
+  }
+
+#undef VAL
+
+  return 1;
+}
+
 static luaL_Reg R[] =
 {
   {"generate_parameters", openssl_dh_generate_parameters},
+  {"problems",            openssl_dh_problems},
 
   {NULL, NULL}
 };
 
+
 int luaopen_dh(lua_State *L)
 {
   auxiliar_newclass(L, "openssl.dh",     dh_funs);
@@ -122,5 +186,7 @@ int luaopen_dh(lua_State *L)
   lua_newtable(L);
   luaL_setfuncs(L, R, 0);
 
+  auxiliar_enumerate(L, -1, dh_problems);
+
   return 1;
 }
