@@ -1238,7 +1238,7 @@ static int openssl_x509_extensions(lua_State* L)
 #else
     int i;
     int n = sk_X509_EXTENSION_num(exts);
-    for (i = 0; i < n; i++)
+    for (i = n - 1; i >= 0; i--)
       sk_X509_EXTENSION_delete(exts, i);
     n = sk_X509_EXTENSION_num(others);
     for (i = 0; i < n; i++)
@@ -1401,7 +1401,6 @@ static luaL_Reg x509_funcs[] =
 
   {"equal",       openssl_x509_equal},
 
-  /* compat with luasec */
   {"digest",     openssl_x509_digest},
   {"extensions", openssl_x509_extensions},
   {"issuer",     openssl_x509_issuer},
@@ -1436,6 +1435,103 @@ static LuaL_Enumeration check_flags_const[] =
 };
 #endif
 
+static LuaL_Enumeration x509_vry_result[] =
+{
+  {"OK",     X509_V_OK},
+#define DEFINE_ENUM(x)  {#x,  X509_V_ERR_##x}
+  DEFINE_ENUM(UNSPECIFIED),
+  DEFINE_ENUM(UNABLE_TO_GET_ISSUER_CERT),
+  DEFINE_ENUM(UNABLE_TO_GET_CRL),
+  DEFINE_ENUM(UNABLE_TO_DECRYPT_CERT_SIGNATURE),
+  DEFINE_ENUM(UNABLE_TO_DECRYPT_CRL_SIGNATURE),
+  DEFINE_ENUM(UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY),
+  DEFINE_ENUM(CERT_SIGNATURE_FAILURE),
+  DEFINE_ENUM(CRL_SIGNATURE_FAILURE),
+  DEFINE_ENUM(CERT_NOT_YET_VALID),
+  DEFINE_ENUM(CERT_HAS_EXPIRED),
+  DEFINE_ENUM(CRL_NOT_YET_VALID),
+  DEFINE_ENUM(CRL_HAS_EXPIRED),
+  DEFINE_ENUM(ERROR_IN_CERT_NOT_BEFORE_FIELD),
+  DEFINE_ENUM(ERROR_IN_CERT_NOT_AFTER_FIELD),
+  DEFINE_ENUM(ERROR_IN_CRL_LAST_UPDATE_FIELD),
+  DEFINE_ENUM(ERROR_IN_CRL_NEXT_UPDATE_FIELD),
+  DEFINE_ENUM(OUT_OF_MEM),
+  DEFINE_ENUM(DEPTH_ZERO_SELF_SIGNED_CERT),
+  DEFINE_ENUM(SELF_SIGNED_CERT_IN_CHAIN),
+  DEFINE_ENUM(UNABLE_TO_GET_ISSUER_CERT_LOCALLY),
+  DEFINE_ENUM(UNABLE_TO_VERIFY_LEAF_SIGNATURE),
+  DEFINE_ENUM(CERT_CHAIN_TOO_LONG),
+  DEFINE_ENUM(CERT_REVOKED),
+  DEFINE_ENUM(INVALID_CA),
+  DEFINE_ENUM(PATH_LENGTH_EXCEEDED),
+  DEFINE_ENUM(INVALID_PURPOSE),
+  DEFINE_ENUM(CERT_UNTRUSTED),
+  DEFINE_ENUM(CERT_REJECTED),
+  DEFINE_ENUM(SUBJECT_ISSUER_MISMATCH),
+  DEFINE_ENUM(AKID_SKID_MISMATCH),
+  DEFINE_ENUM(AKID_ISSUER_SERIAL_MISMATCH),
+  DEFINE_ENUM(KEYUSAGE_NO_CERTSIGN),
+  DEFINE_ENUM(UNABLE_TO_GET_CRL_ISSUER),
+  DEFINE_ENUM(UNHANDLED_CRITICAL_EXTENSION),
+  DEFINE_ENUM(KEYUSAGE_NO_CRL_SIGN),
+  DEFINE_ENUM(UNHANDLED_CRITICAL_CRL_EXTENSION),
+  DEFINE_ENUM(INVALID_NON_CA),
+  DEFINE_ENUM(PROXY_PATH_LENGTH_EXCEEDED),
+  DEFINE_ENUM(KEYUSAGE_NO_DIGITAL_SIGNATURE),
+  DEFINE_ENUM(PROXY_CERTIFICATES_NOT_ALLOWED),
+  DEFINE_ENUM(INVALID_EXTENSION),
+  DEFINE_ENUM(INVALID_POLICY_EXTENSION),
+  DEFINE_ENUM(NO_EXPLICIT_POLICY),
+  DEFINE_ENUM(DIFFERENT_CRL_SCOPE),
+  DEFINE_ENUM(UNSUPPORTED_EXTENSION_FEATURE),
+  DEFINE_ENUM(UNNESTED_RESOURCE),
+  DEFINE_ENUM(PERMITTED_VIOLATION),
+  DEFINE_ENUM(EXCLUDED_VIOLATION),
+  DEFINE_ENUM(SUBTREE_MINMAX),
+  DEFINE_ENUM(APPLICATION_VERIFICATION),
+  DEFINE_ENUM(UNSUPPORTED_CONSTRAINT_TYPE),
+  DEFINE_ENUM(UNSUPPORTED_CONSTRAINT_SYNTAX),
+  DEFINE_ENUM(UNSUPPORTED_NAME_SYNTAX),
+  DEFINE_ENUM(CRL_PATH_VALIDATION_ERROR),
+#if OPENSSL_VERSION_NUMBER > 0x10101000L && !defined(LIBRESSL_VERSION_NUMBER)
+  DEFINE_ENUM(PATH_LOOP),
+  DEFINE_ENUM(SUITE_B_INVALID_VERSION),
+  DEFINE_ENUM(SUITE_B_INVALID_ALGORITHM),
+  DEFINE_ENUM(SUITE_B_INVALID_CURVE),
+  DEFINE_ENUM(SUITE_B_INVALID_SIGNATURE_ALGORITHM),
+  DEFINE_ENUM(SUITE_B_LOS_NOT_ALLOWED),
+  DEFINE_ENUM(SUITE_B_CANNOT_SIGN_P_384_WITH_P_256),
+  DEFINE_ENUM(HOSTNAME_MISMATCH),
+  DEFINE_ENUM(EMAIL_MISMATCH),
+  DEFINE_ENUM(IP_ADDRESS_MISMATCH),
+  DEFINE_ENUM(DANE_NO_MATCH),
+  DEFINE_ENUM(EE_KEY_TOO_SMALL),
+  DEFINE_ENUM(CA_KEY_TOO_SMALL),
+  DEFINE_ENUM(CA_MD_TOO_WEAK),
+  DEFINE_ENUM(INVALID_CALL),
+  DEFINE_ENUM(STORE_LOOKUP),
+  DEFINE_ENUM(NO_VALID_SCTS),
+  DEFINE_ENUM(PROXY_SUBJECT_NAME_VIOLATION),
+  DEFINE_ENUM(OCSP_VERIFY_NEEDED),
+  DEFINE_ENUM(OCSP_VERIFY_FAILED),
+  DEFINE_ENUM(OCSP_CERT_UNKNOWN),
+#ifdef X509_V_ERR_SIGNATURE_ALGORITHM_MISMATCH
+  DEFINE_ENUM(SIGNATURE_ALGORITHM_MISMATCH),
+#endif
+#ifdef X509_V_ERR_NO_ISSUER_PUBLIC_KEY
+  DEFINE_ENUM(NO_ISSUER_PUBLIC_KEY),
+#endif
+#ifdef X509_V_ERR_UNSUPPORTED_SIGNATURE_ALGORITHM
+  DEFINE_ENUM(UNSUPPORTED_SIGNATURE_ALGORITHM),
+#endif
+#ifdef EC_KEY_EXPLICIT_PARAMS
+  DEFINE_ENUM(EC_KEY_EXPLICIT_PARAMS),
+#endif
+#endif
+#undef DEFINE_ENUM
+  {NULL, 0}
+};
+
 int luaopen_x509(lua_State *L)
 {
   auxiliar_newclass(L, "openssl.x509", x509_funcs);
@@ -1466,5 +1562,10 @@ int luaopen_x509(lua_State *L)
   lua_settable(L, -3);
 #endif
 
+  lua_pushliteral(L, "verify_result");
+  lua_newtable(L);
+  auxiliar_enumerate(L, -1, x509_vry_result);
+  lua_settable(L, -3);
+
   return 1;
 }
