@@ -1,6 +1,7 @@
 local lu = require 'luaunit'
 local openssl = require 'openssl'
 local cipher = require'openssl'.cipher
+local helper = require'helper'
 
 TestCipherCompat = {}
 
@@ -42,21 +43,22 @@ function TestCipherCompat:testObject()
 
   local info = obj:info()
   --
-  -- assert(info.name)
   assert(info.block_size)
   assert(info.key_length)
   assert(info.iv_length)
   assert(info.flags)
   assert(info.mode)
 
-  --FIXME:
-  --obj:ctrl
+  if helper.openssl3 then
+    assert(obj:ctrl(openssl.cipher.EVP_CTRL_INIT))
+  else
+    obj:ctrl(openssl.cipher.EVP_CTRL_INIT)
+  end
 
-  --FIXME:
-  --obj:init(self.key)
-  --b = assert(obj:update(self.msg))
-  --b = b .. obj:final()
-  --assert(a==b)
+  obj:init(self.key, self.iv, true)
+  b = assert(obj:update(self.msg))
+  b = b .. obj:final()
+  assert(a==b)
 
   obj1 = cipher.new(self.alg, false, self.key, self.iv)
   b = assert(obj1:update(a))
