@@ -45,7 +45,8 @@ function TestCRL:testNew()
   assert(other:verify(ca.cacert))
   local pem = other:export()
 
-  assert(other:updateTime(os.time(), os.time()+3600))
+  assert(other:updateTime(3600))
+  assert(other:updateTime(os.time(), 3600))
 
   assert(other:extensions({
     openssl.x509.extension.new_extension(
@@ -62,13 +63,10 @@ function TestCRL:testNew()
   assert(other:sign(ca.pkey, ca.cacert:issuer()))
   assert(other:verify(ca.cacert))
   assert(other:verify(ca.pkey))
-  if (other.check) then
-    --FIXME:
-    --assert(other:check(ca.pkey))
-  end
 
   assert(other:export())
   local info = other:parse()
+
   assert(type(info.revoked)=='table')
   assert(type(info.extensions)=='table')
   local t = other:get(0, true)
