@@ -63,6 +63,7 @@ function TestPKCS7:testNew()
   der = assert(p7:export())
   assert(openssl.pkcs7.read(der, 'auto'))
 
+  if not helper.openssl3 then
   p7 = openssl.pkcs7.new()
   p7:add(ca.cacert)
   p7:add(cert)
@@ -76,4 +77,5 @@ function TestPKCS7:testNew()
   local ln, sn = p7:type()
   assert(ln)
   assert(sn)
+  end
 end
