@@ -1,3 +1,11 @@
+--[[
+--2021.09.26 uncommit this block will cause ci fail
+--Should be bug in LuaJIT
+--https://github.com/LuaJIT/LuaJIT/commits/v2.1
+collectgarbage('setpause', 0)
+collectgarbage('setstepmul', 10000000000000)
+--]]
+
 local lu = require'luaunit'
 local openssl = require'openssl'
 
@@ -32,6 +40,8 @@ dofile('8.ssl.lua')
 dofile('9.ocsp.lua')
 dofile('9.srp.lua')
 dofile('9.issue.lua')
+dofile('issue#156.lua')
+dofile('issue#185.lua')
 dofile('dh.lua')
 dofile('dsa.lua')
 dofile('rsa.lua')
@@ -43,10 +53,6 @@ runner:setOutputType("tap")
 local retcode = runner:runSuite()
 print(openssl.errors())
 openssl.clear_error()
---FIXME: libressl gc fail
-local helper = require'helper'
-if not helper.libressl then
-  collectgarbage()
-end
-os.exit(retcode)
+collectgarbage()
+os.exit(retcode, true)
 
